@@ -8,6 +8,7 @@ import pandas as pd
 import datetime
 import logging
 import os
+import re
 
 # move these into local namespace
 import meta as pyhcup_meta
@@ -232,6 +233,7 @@ def long_table_sql(table_name, category, schema=None, ine=True, constraints=['DE
     fields = [
         dict(field='KEY', length=18, data_type='numeric'),
         dict(field='VISITLINK', length=18, data_type='numeric'),
+        dict(field='DAYSTOEVENT', length=18, data_type='numeric'),
         dict(field='YEAR', length=5, data_type='numeric'),
         dict(field='STATE', length=2, data_type='char'),
     ]
@@ -246,6 +248,7 @@ def long_table_sql(table_name, category, schema=None, ine=True, constraints=['DE
             dict(field='CPTHCPCS', length=5, data_type='char'),
             dict(field='CPTMOD1', length=2, data_type='char'),
             dict(field='CPTMOD2', length=2, data_type='char'),
+            dict(field='GROUP_NUMBER', length=5, data_type='numeric'),
         ]
     
     elif category == 'DX':
@@ -256,6 +259,7 @@ def long_table_sql(table_name, category, schema=None, ine=True, constraints=['DE
             dict(field='DXPOA', length=1, data_type='char'),
             dict(field='DXatAdmit', length=1, data_type='char'),
             dict(field='TMDX', length=1, data_type='char'),
+            dict(field='GROUP_NUMBER', length=5, data_type='numeric'),
         ]
     
     elif category == 'PR':
@@ -269,15 +273,23 @@ def long_table_sql(table_name, category, schema=None, ine=True, constraints=['DE
             dict(field='PRV', length=2, data_type='char'),
             dict(field='PCLASS', length=1, data_type='char'),
             dict(field='PRMCCS', length=8, data_type='char'),
+            dict(field='GROUP_NUMBER', length=5, data_type='numeric'),
+        ]
+    
+    elif category == 'UFLAGS':
+        fields += [
+            dict(field='NAME', length=50, data_type='char'),
+            dict(field='VALUE', length=2, data_type='numeric'),
         ]
     
     else:
-        raise Exception("At present only long charges (CHGS), diagnosis (DX), and procedure (PR) category tables are supported. (Got %s)" % category)
+        raise Exception("At present only long charges (CHGS), long diagnosis (DX), long procedure (PR), and long uflag (UFLAGS) category tables are supported. (Got %s)" % category)
 
     fields_df = pd.DataFrame(fields)
     
-    # correct for NaN scale values so column_clause doesn't flip out
-    fields_df.scale = fields_df.scale.map(lambda x: False if pd.isnull(x) else x)
+    if 'scale' in fields_df.columns:
+        # correct for NaN scale values so column_clause doesn't flip out
+        fields_df.scale = fields_df.scale.map(lambda x: False if pd.isnull(x) else x)
     
     sql = table_sql(fields_df, table_name, schema=schema, ine=True)
     
@@ -596,9 +608,13 @@ def pg_wtl_shovel(cnxn, meta_df, category, tbl_source,
     # Get the max number of each variety of column
     maxes = []
     for k in reverse_lm.keys():
-        fields_starting_with = field_s[ field_s.map(lambda x: x[0:len(k)] == k) ]
-        nums = fields_starting_with.map(lambda x: x.split(k)[1])
+        col_name_pattern = k + "\d+"
+        fields_starting_with = field_s[ field_s.map(lambda x: True if re.match(col_name_pattern, x) is not None else False) ]
+        #print "fields_starting_with %s" % fields_starting_with
+        nums = fields_starting_with.map(lambda x: x.split(k)[-1])
+        #print "nums %s" % nums
         maxes.append(nums.astype(int).max())
+        #print "maxes %s" % maxes
 
     # Check to see if we have the same number of each type of column.
     # If not, emit warning but proceed.
@@ -619,6 +635,9 @@ def pg_wtl_shovel(cnxn, meta_df, category, tbl_source,
     elif category == 'DX':
         wnn_base = [x for x in parser.LONG_MAPS[category]['DX']
                     if x in reverse_lm.keys()]
+    elif category == 'PR':
+        wnn_base = [x for x in parser.LONG_MAPS[category]['PR']
+                    if x in reverse_lm.keys()]
     else:
         wnn_base = []
         
@@ -650,6 +669,7 @@ def pg_wtl_shovel(cnxn, meta_df, category, tbl_source,
             for k, v in reverse_lm.iteritems()
             if '%s%s' % (k, x) in meta_df.field.values #if KEYn is a real field
         ]
+        #print "cols_as %s" % cols_as
         
         if isinstance(extra_fields, list) and len(extra_fields) > 0:
             # some extra fields are requested; likely state, year, and key
@@ -698,6 +718,7 @@ def pg_shovel(cnxn, tbl_source, tbl_destination, fields_in, fields_out=False,
         assert len(fields_out) == len(fields_in), \
             "If provided, fields_out must have the same length as fields_in."
         select_fields = ', '.join(['%s AS %s' % cols for cols in zip(fields_in, fields_out)])
+        #print "fields_out %s" % fields_out
         insert_fields = ', '.join(fields_out)
     else:
         select_fields = ', '.join(fields_in)
@@ -780,6 +801,20 @@ def pg_dteload(cnxn, handle, table_name):
     return (table_name, rowcount)
 
 
+def pg_getcols(cnxn, tbl_name):
+    col_sql = """
+    SELECT column_name
+    FROM information_schema.columns
+    WHERE table_name   = %s
+    """
+    
+    cursor = cnxn.cursor()
+    cursor.execute(col_sql, [tbl_name.lower()])
+    result = cursor.fetchall()
+    cols = [x[0] for x in result]
+    return cols
+
+
 def pg_drop(cnxn, tbl_name):
     cursor = cnxn.cursor()
     stmt = 'DROP TABLE IF EXISTS %s;' % tbl_name
